package hashing;

public class ChainedSimpleHashtable<T> implements SimpleHashtable<T> {
	public final static int DEFAULT_TABLESIZE = 100; // 
	  
	private Node<T>[] dataTable;
	private int count;
	
	private class Node<T>{
		private T data;	
	    private Node<T> next;

	    public Node(T data){
	       next = null;
	       this.data = data;
	    }
	}

	public ChainedSimpleHashtable(){
		this(DEFAULT_TABLESIZE);
	}
	
	public ChainedSimpleHashtable(int tablesize){
		dataTable = (Node<T>[])(new Node[tablesize]);
		count=0;
		for(int i=0; i<dataTable.length; i++){
			dataTable[i]=null;			
		}
	}
	
	// Add key
	public void add(T key){
		// add data to table[key.hashCode()%dataTable.length]
		
		int h = Math.abs(key.hashCode()) % dataTable.length;
		Node<T> newnode = new Node<T>(key);
		
		newnode.next=dataTable[h];
		dataTable[h]=newnode;
		count++;
	}
	
	// Return true if key found
	public boolean find(T key){
		int h = Math.abs(key.hashCode()) % dataTable.length;
		
		Node<T> current = dataTable[h];
		while(current!=null){
			if(current.data.equals(key))
				return true;
			current=current.next;
		}
		
		// not found
		return false;
	}
	
	// Remove first occurrence of key, and return true if found
	public boolean remove(T key){
		int h = Math.abs(key.hashCode()) % dataTable.length;
		
		if(dataTable[h]!=null){
			// First item in list
			if(dataTable[h].data.equals(key)){
				dataTable[h]=dataTable[h].next;
				count--;
				return true;
			} else { // rest of list
				Node prev = dataTable[h];
				Node current = dataTable[h].next;
				while(current!=null){
					if(current.data.equals(key)){
						prev.next=current.next;
						count--;
						return true;
					}
					prev=current;
					current=current.next;
				}
			}
		}
		return false;
	}
	
	public String toString(){
		String result="Hashtable storing "+count+" elements in a table of size "+dataTable.length;
		for(int i=0; i<dataTable.length; i++){
			result+="\n"+i+": ";
			Node<T> current=dataTable[i];
			while(current!=null){
				result+=current.data+"->";
				current=current.next;
			}
		}
		return result;	
		
	}
	
}
