package hashing;

public class OpenAddressingSimpleHashtable<T> implements SimpleHashtable<T>{
	
	public final static int DEFAULT_TABLESIZE = 100; 
	public static final int LINEAR=0;
	public static final int QUADRATIC=1;
	
	  
	private T[] dataTable;
	private int count;
	private int probingStrategy;
	
	public OpenAddressingSimpleHashtable(){
		this(DEFAULT_TABLESIZE,LINEAR);
	}

	public OpenAddressingSimpleHashtable(int tablesize){
		this(tablesize,LINEAR);
	}

	public OpenAddressingSimpleHashtable(int tablesize, int probingStrategy){
		dataTable=(T[])(new Object[tablesize]);
		this.probingStrategy=probingStrategy;
		count=0;
		for(int i=0; i<dataTable.length; i++){
			dataTable[i]=null;		 	
		}		
	}
	
	private int probe(int h, int i){
		if(probingStrategy==LINEAR)
			return (h+i)%dataTable.length;
		else // quadratic		
			return (h+i*i)%dataTable.length;
	}
	

	private void expandCapacity(){
		// Keep reference to old dataTable
		T[] oldTable = dataTable;
				
		// Double table size
		dataTable = (T[])(new Object[2*oldTable.length]);
		count=0;

		// Rehash all values in oldTable into the new table
		for (int i=0; i<oldTable.length; i++) {
			if(oldTable[i]!=null)
				add(oldTable[i]);			
		}		
	}
	
	public void add(T key){
		if(2*(count+1)>= dataTable.length) // new load factor >= 0.5
			expandCapacity();

		
		int h = Math.abs(key.hashCode()) % dataTable.length;

		int i=1;
		int current=h;
		while(dataTable[current]!=null){
			current=probe(h,i);
			i++;
		}
			
		dataTable[current]=key;
		count++;		
	}
	
	public boolean find(T key){
		int h = Math.abs(key.hashCode()) % dataTable.length;

		int i=1;
		int current=h;
		while(dataTable[current]!=null && !dataTable[current].equals(key)){
			current=probe(h,i);
			i++;
		}
			
		return (dataTable[current]!=null); // true if non-null
	}

	public boolean remove(T key){
		// Not implemented yet
		return false;
	}
	
	public String toString(){
		String result="Hashtable storing "+count+" elements in a table of size "+dataTable.length;
		for(int i=0; i<dataTable.length; i++){
			result+="\n"+i+": "+dataTable[i];
		}
		return result;
	}

}
